package core.thrift;

import org.apache.commons.pool2.BasePooledObjectFactory;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.thrift.transport.TNonblockingSocket;

import java.net.Socket;


public class ConnectionPoolFactory extends BasePooledObjectFactory<TNonblockingSocket> {
    private String ip;
    private int port;
    private int socketTimeout;

    public ConnectionPoolFactory(String ip, int port, int socketTimeout) {
        this.ip = ip;
        this.port = port;
        this.socketTimeout = socketTimeout;
    }

    @Override
    //创建TNonblockingSocket类型对象方法
    public TNonblockingSocket create() throws Exception {
        return new TNonblockingSocket(ip, port, socketTimeout);
    }

    //把TNonblockingSocket对象打包成池管理的对象PooledObject<TNonblockingSocket>
    @Override
    public PooledObject<TNonblockingSocket> wrap(TNonblockingSocket transport) {
        return new DefaultPooledObject<>(transport);
    }

    /**
     * 验证对象
     */
    @Override
    public boolean validateObject(PooledObject<TNonblockingSocket> p) {
        Socket socket=p.getObject().getSocketChannel().socket();
        boolean closed = socket.isClosed();
        boolean connected = socket.isConnected();
        boolean outputShutdown = socket.isOutputShutdown();
        boolean inputShutdown = socket.isInputShutdown();

        boolean urgentFlag = false;
        try {
            socket.sendUrgentData(0xFF);
            urgentFlag = true;
        } catch (Exception e) {

        }

        return urgentFlag && connected && !closed && !inputShutdown && !outputShutdown;
    }

    /**
     * 销毁对象
     */
    @Override
    public void destroyObject(PooledObject<TNonblockingSocket> p) throws Exception {
        p.getObject().close();
    }
}
