package db.util;

import db.model.bilibili.CidEntity;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.boot.Metadata;
import org.hibernate.boot.MetadataSources;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Projections;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.hibernate.tool.schema.TargetType;
import org.springframework.orm.hibernate5.HibernateTemplate;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

public class ExportUtil {

    private HibernateTemplate template;

    private String fileName;

    private final List<Integer> typeid;

    public ExportUtil(HibernateTemplate template, String fileName) {
        this.template = template;
        this.fileName = fileName;
        this.typeid= (List<Integer>) template.findByCriteria(DetachedCriteria.forClass(CidEntity.class).setProjection(Projections.groupProperty("typeid")));
    }

    public void exportType() throws IOException {
        FileUtils.writeStringToFile(new File(fileName),StringUtils.join(typeid,"\n"),"UTF-8");
    }

    public void exportTableFromDB(Class ...c){
        ServiceRegistry serviceRegistry = new StandardServiceRegistryBuilder().applySettings(template.getSessionFactory().getProperties()).build();
        MetadataSources metadataSources=new MetadataSources(serviceRegistry);
        for(Class t:c){
            metadataSources.addAnnotatedClass(t);
        }
        Metadata metadata = metadataSources.buildMetadata();
        SchemaExport schemaExport = new SchemaExport();
        schemaExport.setOutputFile(fileName);
        schemaExport.setFormat(true);
        schemaExport.create(EnumSet.of(TargetType.SCRIPT), metadata);
    }

    public void exportTable() {
        List<String> tableNames = new ArrayList<>();
        StringBuffer sb = new StringBuffer();

        sb.append("create table data_1\n" +
                "(\n" +
                "\tcid int not null\n" +
                "\t\tprimary key,\n" +
                "\taid int null,\n" +
                "\tauthor varchar(255) null,\n" +
                "\tbackup_vid varchar(255) null,\n" +
                "\tcache varchar(255) null,\n" +
                "\tcover varchar(255) null,\n" +
                "\tdispatch int null,\n" +
                "\tdispatch_servers int null,\n" +
                "\tdp_done tinyint null,\n" +
                "\tdp_done_flv tinyint null,\n" +
                "\tdp_done_hdmp4 tinyint null,\n" +
                "\tdp_done_mp4 tinyint null,\n" +
                "\tduration double null,\n" +
                "\tfiles int null,\n" +
                "\tletv_addr varchar(255) null,\n" +
                "\tletv_vid int null,\n" +
                "\tletv_vu varchar(255) null,\n" +
                "\tmid int null,\n" +
                "\tpage int null,\n" +
                "\tstorage int null,\n" +
                "\tstorage_server int null,\n" +
                "\tsubtitle varchar(255) null,\n" +
                "\ttitle varchar(255) null,\n" +
                "\ttype varchar(255) null,\n" +
                "\tupload int null,\n" +
                "\tupload_meta int null,\n" +
                "\tvid varchar(255) null,\n" +
                "\tvp int null,\n" +
                "\ttype_id int null\n" +
                ")\n" +
                "engine=MyISAM\n" +
                ";\n" +
                "\n");

        for (Integer id : typeid) {
            String tableName = "data_" + id;
            sb.append("create table " + tableName + " like data_1;\n");
            tableNames.add(tableName);
        }

        sb.append("create table data (\n" +
                "       cid integer not null,\n" +
                "        aid integer,\n" +
                "        author varchar(255),\n" +
                "        backup_vid varchar(255),\n" +
                "        cache varchar(255),\n" +
                "        cover varchar(255),\n" +
                "        dispatch integer,\n" +
                "        dispatch_servers integer,\n" +
                "        dp_done tinyint,\n" +
                "        dp_done_flv tinyint,\n" +
                "        dp_done_hdmp4 tinyint,\n" +
                "        dp_done_mp4 tinyint,\n" +
                "        duration double precision,\n" +
                "        files integer,\n" +
                "        letv_addr varchar(255),\n" +
                "        letv_vid integer,\n" +
                "        letv_vu varchar(255),\n" +
                "        mid integer,\n" +
                "        page integer,\n" +
                "        storage integer,\n" +
                "        storage_server integer,\n" +
                "        subtitle varchar(255),\n" +
                "        title varchar(255),\n" +
                "        type varchar(255),\n" +
                "        upload integer,\n" +
                "        upload_meta integer,\n" +
                "        vid varchar(255),\n" +
                "        vp integer,\n" +
                "        type_id integer,\n" +
                "        primary key (cid)\n" +
                "    ) engine=MERGE UNION=(" + StringUtils.join(tableNames, ",") + ") insert_method=no;");


        try {
            FileUtils.writeStringToFile(new File(fileName), sb.toString(), "utf-8", true);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
